<?php
/**
 * Advanced Heading
 *
 * @package ADVHEL
 */
namespace ADV\Heading\Widgets\Advanced_Heading;

use ADV\Heading\Classes\Helper;
use Elementor\Controls_Manager;
use Elementor\Widget_Base;
use Elementor\Group_Control_Border;
use Elementor\Core\Schemes;
use Elementor\Group_Control_Typography;
use Elementor\Group_Control_Background;
use Elementor\Group_Control_Text_Shadow;

defined( 'ABSPATH' ) || die();

class Widget extends Widget_Base {
	
	/**
	 * Get widget name.
	 *
	 * @since 1.0.0
	 * @access public
	 *
	 * @return string Widget name.
	 */
	public function get_name(){
		return 'dl-advanced-heading';
	}

	/**
	 * Get widget title.
	 *
	 * @since 1.0.0
	 * @access public
	 *
	 * @return string Widget title.
	 */
	public function get_title() {
		return esc_html__( 'Advanced Heading', 'adv-heading-elementor' );
	}

	public function get_custom_help_url() {
		return 'https://dynamiclayers.net';
	}

	/**
	 * Get widget icon.
	 *
	 * @since 1.0.0
	 * @access public
	 *
	 * @return string Widget icon.
	 */
	public function get_icon() {
		return 'eicon-heading';
	}

	public function get_categories() {
		return [ 'adv-heading-elementor' ];
	}

	public function get_keywords() {
		return [ 'advance', 'heading', 'title', 'adv-heading-elementor' ];
	}

	public function get_style_depends() {
		return [ 'adv-heading' ];
	}

	/**
	 * Register content related controls
	 */
	protected function register_controls() {

        // Advanced Heading
		$this->start_controls_section(
			'section_heading',
			[
				'label' => esc_html__( 'Advanced Heading', 'adv-heading-elementor' )
			]
		);

		$this->add_control(
            'heading_before',
            [
                'label' => esc_html__( 'Before Text', 'adv-heading-elementor' ),
                'type' => Controls_Manager::TEXT,
                'default' => esc_html__( 'Elementor', 'adv-heading-elementor' ),
                'placeholder' => esc_html__( 'Before Text', 'adv-heading-elementor' ),
				'dynamic' => [
					'active' => true,
				]
            ]
        );

        $this->add_control(
            'heading_center',
            [
                'label' => esc_html__( 'Center Text', 'adv-heading-elementor' ),
                'type' => Controls_Manager::TEXT,
                'default' => esc_html__( 'Advanced', 'adv-heading-elementor' ),
                'placeholder' => esc_html__( 'Center Text', 'adv-heading-elementor' ),
				'dynamic' => [
					'active' => true,
				]
            ]
        );

        $this->add_control(
            'heading_after',
            [
                'label' => esc_html__( 'After Text', 'adv-heading-elementor' ),
                'type' => Controls_Manager::TEXT,
                'default' => esc_html__( 'Heading', 'adv-heading-elementor' ),
                'placeholder' => esc_html__( 'After Text', 'adv-heading-elementor' ),
				'dynamic' => [
					'active' => true,
				]
            ]
        );

		$this->add_control(
			'show_background_text',
			[
				'label' => esc_html__( 'Background Text', 'adv-heading-elementor' ),
				'type' => Controls_Manager::SWITCHER,
				'label_on' => esc_html__( 'Show', 'adv-heading-elementor' ),
				'label_off' => esc_html__( 'Hide', 'adv-heading-elementor' ),
				'return_value' => 'yes',
				'default' => 'no',
                'style_transfer' => true,
			]
		);

		$this->add_control(
			'background_text',
			[
				'label' => esc_html__( 'Text', 'adv-heading-elementor' ),
				'type' => Controls_Manager::TEXT,
				'default' => esc_html__( 'Background', 'adv-heading-elementor' ),
				'placeholder' => esc_html__( 'Background Text', 'adv-heading-elementor' ),
				'condition' => [
					'show_background_text' => 'yes'
				],
				'dynamic' => [
					'active' => true,
				]
			]
		);

        $this->add_control(
            'link',
            [
                'label' => esc_html__( 'Link', 'adv-heading-elementor' ),
                'type' => Controls_Manager::URL,
                'placeholder' => esc_html__( 'https://example.com/', 'adv-heading-elementor' ),
				'separator' => 'after',
				'dynamic' => [
					'active' => true,
				]
            ]
        );

        $this->add_control(
            'title_tag',
            [
                'label' => esc_html__( 'HTML Tag', 'adv-heading-elementor' ),
                'type' => Controls_Manager::CHOOSE,
                'default' => 'h2',
                'options' => [
                    'h1'  => [
                        'title' => esc_html__( 'H1', 'adv-heading-elementor' ),
                        'icon' => 'eicon-editor-h1'
                    ],
                    'h2'  => [
                        'title' => esc_html__( 'H2', 'adv-heading-elementor' ),
                        'icon' => 'eicon-editor-h2'
                    ],
                    'h3'  => [
                        'title' => esc_html__( 'H3', 'adv-heading-elementor' ),
                        'icon' => 'eicon-editor-h3'
                    ],
                    'h4'  => [
                        'title' => esc_html__( 'H4', 'adv-heading-elementor' ),
                        'icon' => 'eicon-editor-h4'
                    ],
                    'h5'  => [
                        'title' => esc_html__( 'H5', 'adv-heading-elementor' ),
                        'icon' => 'eicon-editor-h5'
                    ],
                    'h6'  => [
                        'title' => esc_html__( 'H6', 'adv-heading-elementor' ),
                        'icon' => 'eicon-editor-h6'
                    ]
                ],
                'toggle' => false,
            ]
        );

        $this->add_responsive_control(
            'heading_align',
            [
                'label' => esc_html__( 'Alignment', 'adv-heading-elementor' ),
                'type' => Controls_Manager::CHOOSE,
                'options' => [
                    'left' => [
                        'title' => esc_html__( 'Left', 'adv-heading-elementor' ),
                        'icon' => 'eicon-text-align-left',
                    ],
                    'center' => [
                        'title' => esc_html__( 'Center', 'adv-heading-elementor' ),
                        'icon' => 'eicon-text-align-center',
                    ],
                    'right' => [
                        'title' => esc_html__( 'Right', 'adv-heading-elementor' ),
                        'icon' => 'eicon-text-align-right',
                    ]
                ],
                'default' => 'left',
                'toggle' => false,
                'prefix_class' => 'mp-align-',
                'selectors_dictionary' => [
                    'left' => 'justify-content: flex-start',
                    'center' => 'justify-content: center',
                    'right' => 'justify-content: flex-end',
                ],
                'selectors' => [
                    '{{WRAPPER}} .megapack-heading-tag' => '{{VALUE}}'
                ]
            ]
        );

        $this->add_responsive_control(
            'heading_position',
            [
                'label' => esc_html__( 'Layout', 'adv-heading-elementor' ),
                'type' => Controls_Manager::CHOOSE,
                'options' => [
                    'inline' => [
                        'title' => esc_html__( 'Inline', 'adv-heading-elementor' ),
                        'icon' => 'eicon-ellipsis-h',
                    ],
                    'block' => [
                        'title' => esc_html__( 'Block', 'adv-heading-elementor' ),
                        'icon' => 'eicon-menu-bar',
                    ]
                ],
                'toggle' => false,
                'selectors_dictionary' => [
                    'inline' => 'flex-direction: row',
                    'block' => 'flex-direction: column',
                ],
                'default' => 'inline',
                'prefix_class' => 'mp-layout-',
                'selectors' => [
                    '{{WRAPPER}} .megapack-heading-wrap' => '{{VALUE}}',
                ]
            ]
        );

        $this->end_controls_section();
        
        // Before Text
        $this->start_controls_section(
			'section_before_text',
			[
                'label' => esc_html__( 'Before Text', 'adv-heading-elementor' ),
                'tab'   => Controls_Manager::TAB_STYLE,
			]
        );

        $this->add_group_control(
            Group_Control_Typography::get_type(),
            [
                'name' => 'before_text_typography',
                'selector' => '{{WRAPPER}} .mp-heading-before',
            ]
        );

        $this->add_control(
            'before_text_color',
            [
                'label'     => esc_html__( 'Color', 'adv-heading-elementor' ),
                'type'      => Controls_Manager::COLOR,
                'scheme'    => [
                    'type'  => Schemes\Color::get_type(),
                    'value' => Schemes\Color::COLOR_1,
                ],
                'default'   => '#343352',
                'selectors' => [
                    '{{WRAPPER}} .mp-heading-before' => 'color: {{VALUE}}',
                ],
            ]
        );

        $this->add_responsive_control(
            'before_text_padding',
            [
                'label' => esc_html__( 'Padding', 'adv-heading-elementor' ),
                'type' => Controls_Manager::DIMENSIONS,
                'size_units' => [ 'px', '%' ],
                'selectors' => [
                    '{{WRAPPER}} .mp-heading-before' => 'padding: {{TOP}}{{UNIT}} {{RIGHT}}{{UNIT}} {{BOTTOM}}{{UNIT}} {{LEFT}}{{UNIT}};',
                ],
            ]
        );

        $this->add_responsive_control(
            'before_text_spacing',
            [
                'label' => esc_html__( 'Spacing', 'adv-heading-elementor' ),
                'type' => Controls_Manager::SLIDER,
                'size_units' => [ 'px' ],
                'range' => [
                    'px' => [
                        'min' => -400,
                        'max' => 400,
                    ],
                ],
                'selectors' => [
                    '{{WRAPPER}}.mp-layout-inline .mp-heading-before' => 'margin-right: {{SIZE}}{{UNIT}};',
                    '{{WRAPPER}}.mp-layout-block .mp-heading-before' => 'margin-bottom: {{SIZE}}{{UNIT}};',
                ],
            ]
        );

        $this->add_group_control(
            Group_Control_Border::get_type(),
            [
                'name' => 'before_text_border',
                'selector' => '{{WRAPPER}} .mp-heading-before',
            ]
        );

        $this->add_control(
            'before_text_border_radius',
            [
                'label' => esc_html__( 'Border Radius', 'adv-heading-elementor' ),
                'type' => Controls_Manager::DIMENSIONS,
                'size_units' => [ 'px', '%' ],
                'selectors' => [
                    '{{WRAPPER}} .mp-heading-before' => 'border-radius: {{TOP}}{{UNIT}} {{RIGHT}}{{UNIT}} {{BOTTOM}}{{UNIT}} {{LEFT}}{{UNIT}};',
                ],
            ]
        );

        $this->add_group_control(
            Group_Control_Text_Shadow::get_type(),
            [
                'name' => 'before_text_shadow',
                'label' => esc_html__( 'Text Shadow', 'adv-heading-elementor' ),
                'selector' => '{{WRAPPER}} .mp-heading-before',
            ]
        );

        $this->add_group_control(
            Group_Control_Background::get_type(),
            [
                'name' => 'before_text_background',
                'types' => [ 'classic', 'gradient' ],
                'exclude' => [ 'image' ],
                'selector' => '{{WRAPPER}} .mp-heading-before',
            ]
        );

        $this->add_control(
            'before_text_blend_mode',
            [
                'label' => esc_html__( 'Blend Mode', 'adv-heading-elementor' ),
                'type' => Controls_Manager::SELECT,
                'separator' => 'none',
                'options' => [
                    '' => esc_html__( 'Normal', 'adv-heading-elementor' ),
                    'multiply' => 'Multiply',
                    'screen' => 'Screen',
                    'overlay' => 'Overlay',
                    'darken' => 'Darken',
                    'lighten' => 'Lighten',
                    'color-dodge' => 'Color Dodge',
                    'saturation' => 'Saturation',
                    'color' => 'Color',
                    'difference' => 'Difference',
                    'exclusion' => 'Exclusion',
                    'hue' => 'Hue',
                    'luminosity' => 'Luminosity',
                ],
                'selectors' => [
                    '{{WRAPPER}} .mp-heading-before' => 'mix-blend-mode: {{VALUE}}',
                ],
            ]
        );
        
        $this->end_controls_section();

        // Center Text
        $this->start_controls_section(
			'section_center_text',
			[
                'label' => esc_html__( 'Center Text', 'adv-heading-elementor' ),
                'tab'   => Controls_Manager::TAB_STYLE,
			]
        );

        $this->add_group_control(
            Group_Control_Typography::get_type(),
            [
                'name' => 'center_text_typography',
                'selector' => '{{WRAPPER}} .mp-heading-center',
            ]
        );

        $this->add_control(
            'center_text_color',
            [
                'label'     => esc_html__( 'Color', 'adv-heading-elementor' ),
                'type'      => Controls_Manager::COLOR,
                'scheme'    => [
                    'type'  => Schemes\Color::get_type(),
                    'value' => Schemes\Color::COLOR_1,
                ],
                'default'   => '#343352',
                'selectors' => [
                    '{{WRAPPER}} .mp-heading-center' => 'color: {{VALUE}}',
                ],
            ]
        );

        $this->add_responsive_control(
            'center_text_padding',
            [
                'label' => esc_html__( 'Padding', 'adv-heading-elementor' ),
                'type' => Controls_Manager::DIMENSIONS,
                'size_units' => [ 'px', '%' ],
                'selectors' => [
                    '{{WRAPPER}} .mp-heading-center' => 'padding: {{TOP}}{{UNIT}} {{RIGHT}}{{UNIT}} {{BOTTOM}}{{UNIT}} {{LEFT}}{{UNIT}};',
                ],
            ]
        );

        $this->add_responsive_control(
            'center_text_spacing',
            [
                'label' => esc_html__( 'Spacing', 'adv-heading-elementor' ),
                'type' => Controls_Manager::SLIDER,
                'size_units' => [ 'px' ],
                'range' => [
                    'px' => [
                        'min' => -400,
                        'max' => 400,
                    ],
                ],
                'selectors' => [
                    '{{WRAPPER}}.mp-layout-inline .mp-heading-center' => 'margin-right: {{SIZE}}{{UNIT}};',
                    '{{WRAPPER}}.mp-layout-block .mp-heading-center' => 'margin-bottom: {{SIZE}}{{UNIT}};',
                ],
            ]
        );

        $this->add_group_control(
            Group_Control_Border::get_type(),
            [
                'name' => 'center_text_border',
                'selector' => '{{WRAPPER}} .mp-heading-center',
            ]
        );

        $this->add_control(
            'center_text_border_radius',
            [
                'label' => esc_html__( 'Border Radius', 'adv-heading-elementor' ),
                'type' => Controls_Manager::DIMENSIONS,
                'size_units' => [ 'px', '%' ],
                'selectors' => [
                    '{{WRAPPER}} .mp-heading-center' => 'border-radius: {{TOP}}{{UNIT}} {{RIGHT}}{{UNIT}} {{BOTTOM}}{{UNIT}} {{LEFT}}{{UNIT}};',
                ],
            ]
        );

        $this->add_group_control(
            Group_Control_Text_Shadow::get_type(),
            [
                'name' => 'center_text_shadow',
                'label' => esc_html__( 'Text Shadow', 'adv-heading-elementor' ),
                'selector' => '{{WRAPPER}} .mp-heading-center',
            ]
        );

        $this->add_group_control(
            Group_Control_Background::get_type(),
            [
                'name' => 'center_text_background',
                'types' => [ 'classic', 'gradient' ],
                'exclude' => [ 'image' ],
                'selector' => '{{WRAPPER}} .mp-heading-center',
            ]
        );

        $this->add_control(
            'center_text_blend_mode',
            [
                'label' => esc_html__( 'Blend Mode', 'adv-heading-elementor' ),
                'type' => Controls_Manager::SELECT,
                'separator' => 'none',
                'options' => [
                    '' => esc_html__( 'Normal', 'adv-heading-elementor' ),
                    'multiply' => 'Multiply',
                    'screen' => 'Screen',
                    'overlay' => 'Overlay',
                    'darken' => 'Darken',
                    'lighten' => 'Lighten',
                    'color-dodge' => 'Color Dodge',
                    'saturation' => 'Saturation',
                    'color' => 'Color',
                    'difference' => 'Difference',
                    'exclusion' => 'Exclusion',
                    'hue' => 'Hue',
                    'luminosity' => 'Luminosity',
                ],
                'selectors' => [
                    '{{WRAPPER}} .mp-heading-center' => 'mix-blend-mode: {{VALUE}}',
                ],
            ]
        );

        $this->end_controls_section();

        // After Text
        $this->start_controls_section(
			'section_after_text',
			[
                'label' => esc_html__( 'After Text', 'adv-heading-elementor' ),
                'tab'   => Controls_Manager::TAB_STYLE,
			]
        );

        $this->add_group_control(
            Group_Control_Typography::get_type(),
            [
                'name' => 'after_text_typography',
                'selector' => '{{WRAPPER}} .mp-heading-after',
            ]
        );

        $this->add_control(
            'after_text_color',
            [
                'label'     => esc_html__( 'Color', 'adv-heading-elementor' ),
                'type'      => Controls_Manager::COLOR,
                'scheme'    => [
                    'type'  => Schemes\Color::get_type(),
                    'value' => Schemes\Color::COLOR_1,
                ],
                'default'   => '#db3157',
                'selectors' => [
                    '{{WRAPPER}} .mp-heading-after' => 'color: {{VALUE}}',
                ],
            ]
        );

        $this->add_responsive_control(
            'after_text_padding',
            [
                'label' => esc_html__( 'Padding', 'adv-heading-elementor' ),
                'type' => Controls_Manager::DIMENSIONS,
                'size_units' => [ 'px', '%' ],
                'selectors' => [
                    '{{WRAPPER}} .mp-heading-after' => 'padding: {{TOP}}{{UNIT}} {{RIGHT}}{{UNIT}} {{BOTTOM}}{{UNIT}} {{LEFT}}{{UNIT}};',
                ],
            ]
        );

        $this->add_responsive_control(
            'after_text_spacing',
            [
                'label' => esc_html__( 'Spacing', 'adv-heading-elementor' ),
                'type' => Controls_Manager::SLIDER,
                'size_units' => [ 'px' ],
                'range' => [
                    'px' => [
                        'min' => -400,
                        'max' => 400,
                    ],
                ],
                'selectors' => [
                    '{{WRAPPER}}.mp-layout-inline .mp-heading-after' => 'margin-right: {{SIZE}}{{UNIT}};',
                    '{{WRAPPER}}.mp-layout-block .mp-heading-after' => 'margin-bottom: {{SIZE}}{{UNIT}};',
                ],
            ]
        );

        $this->add_group_control(
            Group_Control_Border::get_type(),
            [
                'name' => 'after_text_border',
                'selector' => '{{WRAPPER}} .mp-heading-after',
            ]
        );

        $this->add_control(
            'after_text_border_radius',
            [
                'label' => esc_html__( 'Border Radius', 'adv-heading-elementor' ),
                'type' => Controls_Manager::DIMENSIONS,
                'size_units' => [ 'px', '%' ],
                'selectors' => [
                    '{{WRAPPER}} .mp-heading-after' => 'border-radius: {{TOP}}{{UNIT}} {{RIGHT}}{{UNIT}} {{BOTTOM}}{{UNIT}} {{LEFT}}{{UNIT}};',
                ],
            ]
        );

        $this->add_group_control(
            Group_Control_Text_Shadow::get_type(),
            [
                'name' => 'after_text_shadow',
                'label' => esc_html__( 'Text Shadow', 'adv-heading-elementor' ),
                'selector' => '{{WRAPPER}} .mp-heading-after',
            ]
        );

        $this->add_group_control(
            Group_Control_Background::get_type(),
            [
                'name' => 'after_text_background',
                'types' => [ 'classic', 'gradient' ],
                'exclude' => [ 'image' ],
                'selector' => '{{WRAPPER}} .mp-heading-after',
            ]
        );

        $this->add_control(
            'after_text_blend_mode',
            [
                'label' => esc_html__( 'Blend Mode', 'adv-heading-elementor' ),
                'type' => Controls_Manager::SELECT,
                'separator' => 'none',
                'options' => [
                    '' => esc_html__( 'Normal', 'adv-heading-elementor' ),
                    'multiply' => 'Multiply',
                    'screen' => 'Screen',
                    'overlay' => 'Overlay',
                    'darken' => 'Darken',
                    'lighten' => 'Lighten',
                    'color-dodge' => 'Color Dodge',
                    'saturation' => 'Saturation',
                    'color' => 'Color',
                    'difference' => 'Difference',
                    'exclusion' => 'Exclusion',
                    'hue' => 'Hue',
                    'luminosity' => 'Luminosity',
                ],
                'selectors' => [
                    '{{WRAPPER}} .mp-heading-after' => 'mix-blend-mode: {{VALUE}}',
                ],
            ]
        );

        $this->end_controls_section();

        // Custom Border
        $this->start_controls_section(
			'section_custom_border',
			[
                'label' => esc_html__( 'Custom Border', 'adv-heading-elementor' ),
                'tab'   => Controls_Manager::TAB_STYLE,
			]
        );

        $this->add_control(
            'border_type',
            [
                'label' => esc_html__( 'Border Type', 'adv-heading-elementor' ),
                'type' => Controls_Manager::SELECT,
                'options' => [
                    'none' => esc_html__( 'None', 'adv-heading-elementor' ),
                    'solid' => esc_html__( 'Solid', 'adv-heading-elementor' ),
                    'double' => esc_html__( 'Double', 'adv-heading-elementor' ),
                    'dotted' => esc_html__( 'Dotted', 'adv-heading-elementor' ),
                    'dashed' => esc_html__( 'Dashed', 'adv-heading-elementor' ),
                    'groove' => esc_html__( 'Groove', 'adv-heading-elementor' ),
                ],
                'default' => 'none',
                'selectors' => [
                    '{{WRAPPER}} .mp-border:after' => 'border-bottom-style: {{VALUE}};',
                ],
            ]
        );

        $this->add_responsive_control(
            'border_length',
            [
                'label' => esc_html__( 'Length', 'adv-heading-elementor' ),
                'type' => Controls_Manager::SLIDER,
                'size_units' => [ 'px' ],
                'range' => [
                    'px' => [
                        'min' => 1,
                        'max' => 800,
                    ],
                ],
                'condition' => [
                    'border_type!' => 'none',
                ],
                'selectors' => [
                    '{{WRAPPER}} .mp-border:after' => 'width: {{SIZE}}{{UNIT}};',
                ],
            ]
        );

        $this->add_responsive_control(
            'border_width',
            [
                'label' => esc_html__( 'Width', 'adv-heading-elementor' ),
                'type' => Controls_Manager::SLIDER,
                'default' => [
                    'unit' => 'px',
                    'size' => 3
                ],
                'range' => [
                    'px' => [
                        'min' => 1,
                        'max' => 100,
                    ],
                ],
                'condition' => [
                    'border_type!' => 'none',
                ],
                'selectors' => [
                    '{{WRAPPER}} .mp-border:after' => 'border-bottom-width: {{SIZE}}{{UNIT}};',
                ],
            ]
        );

        $this->add_control(
            'border_color',
            [
                'label' => esc_html__( 'Color', 'adv-heading-elementor' ),
                'type' => Controls_Manager::COLOR,
                'condition' => [
                    'border_type!' => 'none',
                ],
                'selectors' => [
                    '{{WRAPPER}} .mp-border:after' => 'border-color: {{VALUE}}',
                ],
            ]
        );

        $this->add_control(
            'border_radius',
            [
                'label' => esc_html__( 'Border radius', 'adv-heading-elementor' ),
                'type' => Controls_Manager::DIMENSIONS,
                'size_units' => [ 'px', '%' ],
                'condition' => [
                    'border_type!' => 'none',
                ],
                'selectors' => [
                    '{{WRAPPER}} .mp-border:after' => 'border-radius: {{TOP}}{{UNIT}} {{RIGHT}}{{UNIT}} {{BOTTOM}}{{UNIT}} {{LEFT}}{{UNIT}};'
                ],
            ]
        );

        $this->add_control(
            'border_offset_toggle',
            [
                'label' => esc_html__( 'Offset', 'adv-heading-elementor' ),
                'type' => Controls_Manager::POPOVER_TOGGLE,
                'label_off' => esc_html__( 'None', 'adv-heading-elementor' ),
                'label_on' => esc_html__( 'Custom', 'adv-heading-elementor' ),
                'return_value' => 'yes',
                'condition' => [
                    'border_type!' => 'none',
                ],
            ]
        );

        $this->start_popover();

        $this->add_responsive_control(
            'border_horizontal_position',
            [
                'label' => esc_html__( 'Horizontal Position', 'adv-heading-elementor' ),
                'type' => Controls_Manager::SLIDER,
                'size_units' => [ '%' ],
                'default' => [
                    'unit' => '%',
                    'size' => 0
                ],
                'range' => [
                    '%' => [
                        'min' => -100,
                        'max' => 100,
                    ],
                ],
                'condition' => [
                   'border_offset_toggle' => 'yes'
                ],
                'selectors' => [
                    '{{WRAPPER}} .mp-border:after' => 'left: {{SIZE}}{{UNIT}};',
                ],
            ]
        );

        $this->add_responsive_control(
            'border_vertical_position',
            [
                'label' => esc_html__( 'Vertical Position', 'adv-heading-elementor' ),
                'type' => Controls_Manager::SLIDER,
                'size_units' => [ '%' ],
                'default' => [
                    'unit' => '%',
                    'size' => 0
                ],
                'range' => [
                    '%' => [
                        'min' => -100,
                        'max' => 100,
                    ],
                ],
                'condition' => [
                    'border_offset_toggle' => 'yes'
                ],
                'selectors' => [
                    '{{WRAPPER}} .mp-border:after' => 'bottom: {{SIZE}}{{UNIT}};',
                ],
            ]
        );

        $this->end_popover();

        $this->end_controls_section();

         // Background Text
         $this->start_controls_section(
			'section_bg_text',
			[
                'label' => esc_html__( 'Background Text', 'adv-heading-elementor' ),
                'tab'   => Controls_Manager::TAB_STYLE,
			]
        );

        $this->add_control(
            'background_note',
            [
                'label' => false,
                'type' => Controls_Manager::RAW_HTML,
                'raw' => esc_html__( '<strong>Background Text</strong> is Hidden on Content Tab', 'adv-heading-elementor' ),
                'condition' => [
                    'show_background_text!' => 'yes',
                ],
            ]
        );

        $this->add_control(
            'background_text_color',
            [
                'label' => esc_html__( 'Color', 'adv-heading-elementor' ),
                'type' => Controls_Manager::COLOR,
                'condition' => [
                    'show_background_text' => 'yes',
                ],
                'selectors' => [
                    '{{WRAPPER}} .megapack-heading-wrap:before' => 'color: {{VALUE}}',
                ],
            ]
        );

        $this->add_group_control(
            Group_Control_Typography::get_type(),
            [
                'name' => 'background_text_typography',
                'selector' => '{{WRAPPER}} .megapack-heading-wrap:before',
                'condition' => [
                    'show_background_text' => 'yes',
                ],
            ]
        );

        $this->add_control(
            'background_offset_toggle',
            [
                'label' => esc_html__( 'Offset', 'adv-heading-elementor' ),
                'type' => Controls_Manager::POPOVER_TOGGLE,
                'label_off' => esc_html__( 'None', 'adv-heading-elementor' ),
                'label_on' => esc_html__( 'Custom', 'adv-heading-elementor' ),
                'return_value' => 'yes',
                'condition' => [
                    'show_background_text' => 'yes',
                ],
            ]
        );

        $this->start_popover();

        $this->add_responsive_control(
            'background_horizontal_position',
            [
                'label' => esc_html__( 'Horizontal Position', 'adv-heading-elementor' ),
                'type' => Controls_Manager::SLIDER,
                'size_units' => [ 'px', '%' ],
                'default' => [
                    'unit' => '%',
                ],
                'range' => [
                    '%' => [
                        'min' => -100,
                        'max' => 100,
                    ],
                ],
                'condition' => [
                    'background_offset_toggle' => 'yes'
                ],
                'selectors' => [
                    '{{WRAPPER}} .megapack-heading-wrap:before' => 'left: {{SIZE}}{{UNIT}};',
                ],
            ]
        );

        $this->add_responsive_control(
            'background_vertical_position',
            [
                'label' => esc_html__( 'Vertical Position', 'adv-heading-elementor' ),
                'type' => Controls_Manager::SLIDER,
                'size_units' => [ 'px', '%' ],
                'default' => [
                    'unit' => '%',
                ],
                'range' => [
                    '%' => [
                        'min' => -100,
                        'max' => 200,
                    ],
                ],
                'condition' => [
                    'background_offset_toggle' => 'yes'
                ],
                'selectors' => [
                    '{{WRAPPER}} .megapack-heading-wrap:before' => 'top: {{SIZE}}{{UNIT}};',
                ],
            ]
        );

        $this->end_popover();

        $this->end_controls_section();
	}

	protected function render(){
		$settings = $this->get_settings_for_display();
        $has_link = false;

		if ( ! empty( $settings['link']['url'] ) ) {
			$this->add_link_attributes( 'link', $settings['link'] );
			$has_link = true;
        }
        $this->add_render_attribute( 'heading_before', [ 'class' => 'mp-heading-before' ] );
        $this->add_render_attribute( 'heading_center', [ 'class' => 'mp-heading-center' ] );
        $this->add_render_attribute( 'heading_after', [ 'class' => 'mp-heading-after' ] );
        $this->add_inline_editing_attributes( 'heading_before', 'none' );
        $this->add_inline_editing_attributes( 'heading_center', 'none' );
        $this->add_inline_editing_attributes( 'heading_after', 'none' );
    ?>
        <<?php echo esc_attr( $settings['title_tag'] ); ?> class="megapack-heading-tag">
        <?php if( $has_link ) : ?>
            <a <?php $this->print_render_attribute_string( 'link' ) ?>>
        <?php endif; ?>
            <div class="megapack-heading-wrap" data-bg-text="<?php echo esc_attr( $settings['background_text'] ); ?>">
                <?php if( ! empty( $settings[ 'heading_before' ] ) ) : ?>
                <span <?php echo $this->get_render_attribute_string( 'heading_before' ); ?>><?php echo Helper::kses_basic( $settings[ 'heading_before' ] ); ?></span>
                <?php endif; ?>
                <?php if( ! empty( $settings[ 'heading_center' ] ) ) : ?>
                <span <?php echo $this->get_render_attribute_string( 'heading_center' ); ?>><?php echo Helper::kses_basic( $settings[ 'heading_center' ] ); ?></span>
                <?php endif; ?>
                <?php if( ! empty( $settings[ 'heading_after' ] ) ) : ?>
                <span <?php echo $this->get_render_attribute_string( 'heading_after' ); ?>><?php echo Helper::kses_basic( $settings[ 'heading_after' ] ); ?></span>
                <?php endif; ?>
                <?php if( 'none' != $settings['border_type'] ) : ?>
                <span class="mp-border"></span>
                <?php endif; ?>
            </div>
        <?php if( $has_link ) : ?>
            </a>
        <?php endif; ?>
        </<?php echo esc_attr( $settings['title_tag'] ); ?>>
	<?php
    }
    
    protected function content_template() {
        ?>
        <#
        var has_link = false;

        if( '' != settings.link.url ) {
            var has_link = true;
        }

        var target = settings.link.is_external ? ' target="_blank"' : '';
        var nofollow = settings.link.nofollow ? ' rel="nofollow"' : '';
        var bg_text = settings.show_background_text === 'yes' ? settings.background_text : '';

        view.addRenderAttribute( 'heading_before', 'class', [ 'mp-heading-before' ] );
        view.addRenderAttribute( 'heading_center', 'class', [ 'mp-heading-center' ] );
        view.addRenderAttribute( 'heading_after', 'class', [ 'mp-heading-after' ] );

        view.addInlineEditingAttributes( 'heading_before', 'none' );
		view.addInlineEditingAttributes( 'heading_center', 'none' );
		view.addInlineEditingAttributes( 'heading_after', 'none' );
        #>
        <{{{ settings.title_tag }}} class="megapack-heading-tag">
            <# if( has_link ){ #>
            <a href="{{ settings.link.url }}"{{ target }}{{ nofollow }}>
            <# } #>
            <div class="megapack-heading-wrap" data-bg-text="{{{ bg_text }}}">
                <# if( '' != settings.heading_before ) { #>
                <span {{{ view.getRenderAttributeString( 'heading_before' ) }}}>{{{ settings.heading_before }}}</span>
                <# } #>
                <# if( '' != settings.heading_center ) { #>
                <span {{{ view.getRenderAttributeString( 'heading_center' ) }}}>{{{ settings.heading_center }}}</span>
                <# } #>
                <# if( '' != settings.heading_after ) { #>
                <span {{{ view.getRenderAttributeString( 'heading_after' ) }}}>{{{ settings.heading_after }}}</span>
                <# } #>
                <# if( 'none' != settings.border_type ) { #>
                <span class="mp-border"></span>
                <# } #>
            </div>
            <# if( has_link ){ #>
            </a>
            <# } #>
        </{{{ settings.title_tag }}}>
        <?php
    }
}