<?php
/**
 * Animated Heading
 *
 * @package ADVHEL
 */
namespace ADV\Heading\Widgets\Animated_Heading;

use Elementor\Controls_Manager;
use Elementor\Group_Control_Text_Shadow;
use Elementor\Group_Control_Typography;
use Elementor\Repeater;
use Elementor\Widget_Base;

defined( 'ABSPATH' ) || die();

class Widget extends Widget_Base {

    public function __construct( $data = [], $args = null ) {
		parent::__construct($data, $args);
		$min = ( WP_DEBUG === true ) ? '' : '.min';
        wp_register_script( 'adv-animated-text', ADVHEL_ASSETS . '/js/animated-text.min.js', [ 'elementor-frontend' ], '1.0.0', true );
        wp_register_script( 'adv-heading-script', ADVHEL_ASSETS . '/js/scripts' . $min . '.js', [ 'elementor-frontend', 'adv-animated-text' ], '1.0.1', true );
	}

    /**
     * Get widget name.
     *
     * @since 1.0.0
     * @access public
     *
     * @return string Widget name.
     */
    public function get_name() {
        return 'dl-animated-heading';
    }

    /**
     * Get widget title.
     *
     * @since 1.0.0
     * @access public
     *
     * @return string Widget title.
     */
    public function get_title() {
        return esc_html__( 'Animated Heading', 'adv-heading-elementor' );
    }

    public function get_custom_help_url() {
        return 'https://dynamiclayers.net';
    }

    /**
     * Get widget icon.
     *
     * @since 1.0.0
     * @access public
     *
     * @return string Widget icon.
     */
    public function get_icon() {
        return 'eicon-animation-text';
    }

    public function get_categories() {
        return ['adv-heading-elementor'];
    }

    public function get_keywords() {
        return ['animated', 'text', 'heading', 'adv-heading-elementor'];
    }

    public function get_script_depends() {
        return [ 'adv-animated-text', 'adv-heading-script' ];
    }

    public function get_style_depends() {
        return ['adv-animated-heading'];
    }

    /**
     * Register controls
     */
    protected function register_controls() {

        // Content
        $this->start_controls_section(
            'section_content',
            [
                'label' => esc_html__( 'Content', 'adv-heading-elementor' ),
            ]
        );

        $this->add_control(
            'before_text',
            [
                'label'       => esc_html__( 'Before Text', 'adv-heading-elementor' ),
                'type'        => Controls_Manager::TEXT,
                'default'     => esc_html__( 'DynamicLayers is', 'adv-heading-elementor' ),
                'placeholder' => esc_html__( 'Before Text', 'adv-heading-elementor' ),
                'dynamic'     => [
                    'active' => true,
                ],
            ]
        );

        $this->add_control(
            'after_text',
            [
                'label'       => esc_html__( 'After Text', 'adv-heading-elementor' ),
                'type'        => Controls_Manager::TEXT,
                'default'     => '',
                'placeholder' => esc_html__( 'After Text', 'adv-heading-elementor' ),
                'dynamic'     => [
                    'active' => true,
                ],
            ]
        );

        $repeater = new Repeater();

        $repeater->add_control(
            'text', [
                'label'   => esc_html__( 'Text', 'adv-heading-elementor' ),
                'type'    => Controls_Manager::TEXT,
                'default' => esc_html__( 'Awesome', 'adv-heading-elementor' ),
                'dynamic' => [
                    'active' => true,
                ],
            ]
        );

        $repeater->add_control(
            'text_customize',
            [
                'label'          => esc_html__( 'Want To Customize Text?', 'adv-heading-elementor' ),
                'type'           => Controls_Manager::SWITCHER,
                'label_on'       => esc_html__( 'Yes', 'adv-heading-elementor' ),
                'label_off'      => esc_html__( 'No', 'adv-heading-elementor' ),
                'return_value'   => 'yes',
                'style_transfer' => true,
            ]
        );

        $repeater->add_group_control(
            Group_Control_Typography::get_type(),
            [
                'name'           => 'text_typography',
                'label'          => esc_html__( 'Typography', 'adv-heading-elementor' ),
                'exclude'        => [
                    'font_size',
                    'line_height',
                ],
                'selector'       => '{{WRAPPER}} .dl-animated-text-wrap .dl-animated-text > {{CURRENT_ITEM}}, {{WRAPPER}} .dl-animated-text-wrap .dl-animated-text > {{CURRENT_ITEM}} i, {{WRAPPER}} .dl-animated-text-wrap .dl-animated-text > {{CURRENT_ITEM}} em',
                'condition'      => [
                    'text_customize' => 'yes',
                ],
                'style_transfer' => true,
            ]
        );

        $repeater->add_control(
            'text_color',
            [
                'label'          => esc_html__( 'Color', 'adv-heading-elementor' ),
                'type'           => Controls_Manager::COLOR,
                'selectors'      => [
                    '{{WRAPPER}} .dl-animated-text-wrap .dl-animated-text > {{CURRENT_ITEM}} '     => 'color: {{VALUE}}; -webkit-background-clip: initial; -webkit-text-fill-color:initial; background: none;',
                    '{{WRAPPER}} .dl-animated-text-wrap .dl-animated-text > {{CURRENT_ITEM}} i'    => 'color: {{VALUE}}; -webkit-background-clip: initial; -webkit-text-fill-color:initial; background: none;',
                    '{{WRAPPER}} .dl-animated-text-wrap .dl-animated-text > {{CURRENT_ITEM}} i em' => 'color: {{VALUE}}; -webkit-background-clip: initial; -webkit-text-fill-color:initial; background: none;',
                ],
                'condition'      => [
                    'text_customize' => 'yes',
                ],
                'style_transfer' => true,
            ]
        );

        $repeater->add_group_control(
            Group_Control_Text_Shadow::get_type(),
            [
                'name'           => 'text_shadow',
                'label'          => esc_html__( 'Text Shadow', 'adv-heading-elementor' ),
                'selector'       => '{{WRAPPER}} .dl-animated-text > {{CURRENT_ITEM}}',
                'condition'      => [
                    'text_customize' => 'yes',
                ],
                'style_transfer' => true,
            ]
        );

        $this->add_control(
            'animated_text',
            [
                'label'       => esc_html__( 'Animated Text', 'adv-heading-elementor' ),
                'type'        => Controls_Manager::REPEATER,
                'fields'      => $repeater->get_controls(),
                'default'     => [
                    [
                        'text' => esc_html__( 'Awesome', 'adv-heading-elementor' ),
                    ],
                    [
                        'text' => esc_html__( 'Cool', 'adv-heading-elementor' ),
                    ],
                    [
                        'text' => esc_html__( 'Nice', 'adv-heading-elementor' ),
                    ],
                ],
                'title_field' => '{{{ text }}}',
            ]
        );

        $this->add_control(
            'hr',
            [
                'type' => Controls_Manager::DIVIDER,
            ]
        );

        $this->add_control(
            'html_tag',
            [
                'label'   => esc_html__( 'HTML Tag', 'adv-heading-elementor' ),
                'type'    => Controls_Manager::SELECT,
                'options' => [
                    'h1' => esc_html__( 'H1', 'adv-heading-elementor' ),
                    'h2' => esc_html__( 'H2', 'adv-heading-elementor' ),
                    'h3' => esc_html__( 'H3', 'adv-heading-elementor' ),
                    'h4' => esc_html__( 'H4', 'adv-heading-elementor' ),
                    'h5' => esc_html__( 'H5', 'adv-heading-elementor' ),
                    'h6' => esc_html__( 'H6', 'adv-heading-elementor' ),
                    'p'  => esc_html__( 'P', 'adv-heading-elementor' ),
                ],
                'default' => 'h2',
            ]
        );

        $this->add_responsive_control(
            'align',
            [
                'label'     => esc_html__( 'Alignment', 'adv-heading-elementor' ),
                'type'      => Controls_Manager::CHOOSE,
                'options'   => [
                    'left'    => [
                        'title' => esc_html__( 'Left', 'adv-heading-elementor' ),
                        'icon'  => 'eicon-text-align-left',
                    ],
                    'center'  => [
                        'title' => esc_html__( 'Center', 'adv-heading-elementor' ),
                        'icon'  => 'eicon-text-align-center',
                    ],
                    'right'   => [
                        'title' => esc_html__( 'Right', 'adv-heading-elementor' ),
                        'icon'  => 'eicon-text-align-right',
                    ],
                    'justify' => [
                        'title' => esc_html__( 'Justify', 'adv-heading-elementor' ),
                        'icon'  => 'eicon-text-align-justify',
                    ],
                ],
                'toggle'    => true,
                'selectors' => [
                    '{{WRAPPER}} .cd-headline' => 'text-align: {{VALUE}}',
                ],
            ]
        );

        $this->end_controls_section();

        // Animation Settings
        $this->start_controls_section(
            'section_animation_settings',
            [
                'label'      => esc_html__( 'Animation Settings', 'adv-heading-elementor' ),
                'tab'        => Controls_Manager::TAB_CONTENT
            ]
        );

        $this->add_control(
            'animation_type',
            [
                'label'          => esc_html__( 'Animation Type', 'adv-heading-elementor' ),
                'type'           => Controls_Manager::SELECT,
                'default'        => 'letters type',
                'options'        => [
                    'rotate-1'         => esc_html__( 'Rotate 1', 'adv-heading-elementor' ),
                    'letters rotate-2' => esc_html__( 'Rotate 2', 'adv-heading-elementor' ),
                    'letters rotate-3' => esc_html__( 'Rotate 3', 'adv-heading-elementor' ),
                    'letters type'     => esc_html__( 'Type', 'adv-heading-elementor' ),
                    'slide'            => esc_html__( 'Slide', 'adv-heading-elementor' ),
                    'clip'             => esc_html__( 'Clip', 'adv-heading-elementor' ),
                    'zoom'             => esc_html__( 'Zoom', 'adv-heading-elementor' ),
                    'letters scale'    => esc_html__( 'Scale', 'adv-heading-elementor' ),
                    'push'             => esc_html__( 'Push', 'adv-heading-elementor' ),
                    'loading-bar'      => esc_html__( 'Loading Bar', 'adv-heading-elementor' ),
                ],
                'style_transfer' => true,
            ]
        );

        $this->add_control(
            'animation_delay',
            [
                'label'              => esc_html__( 'Animation Delay', 'adv-heading-elementor' ),
                'type'               => Controls_Manager::NUMBER,
                'min'                => 1000,
                'step'               => 100,
                'max'                => 30000,
                'default'            => 2500,
                'description'        => esc_html__( 'Animation Delay in milliseconds. Min 1000 and Max 30000.', 'adv-heading-elementor' ),
                'frontend_available' => true,
            ]
        );

        $this->add_control(
            'clip_anim_duration',
            [
                'label'              => esc_html__( 'Reveal Duration', 'adv-heading-elementor' ),
                'type'               => Controls_Manager::NUMBER,
                'min'                => 100,
                'step'               => 100,
                'max'                => 10000,
                'default'            => 600,
                'description'        => esc_html__( 'Reveal Duration in milliseconds. Min 100 and Max 10000.', 'adv-heading-elementor' ),
                'frontend_available' => true,
                'condition'    => [
                   'animation_type'    => 'clip',
                ],
            ]
        );

        $this->add_control(
            'clip_anim_delay',
            [
                'label'              => esc_html__( 'Reveal Animation Delay', 'adv-heading-elementor' ),
                'type'               => Controls_Manager::NUMBER,
                'min'                => 100,
                'step'               => 100,
                'max'                => 10000,
                'default'            => 1500,
                'description'        => esc_html__( 'Reveal Animation Delay in milliseconds. Min 100 and Max 10000.', 'adv-heading-elementor' ),
                'frontend_available' => true,
                'condition'    => [
                   'animation_type'    => 'clip',
                ],
            ]
        );

        $this->add_control(
            'type_letter_delay',
            [
                'label'              => esc_html__( 'Type Letter Delay', 'adv-heading-elementor' ),
                'type'               => Controls_Manager::NUMBER,
                'min'                => 100,
                'step'               => 100,
                'max'                => 5000,
                'default'            => 150,
                'description'        => esc_html__( 'Type Letter Delay in milliseconds. Min 100 and Max 10000.', 'adv-heading-elementor' ),
                'frontend_available' => true,
                'condition'    => [
                    'animation_type'    => 'letters type',
                ],
            ]
        );

        $this->add_control(
            'selection_duration',
            [
                'label'              => esc_html__( 'Selection Duration', 'adv-heading-elementor' ),
                'type'               => Controls_Manager::NUMBER,
                'min'                => 100,
                'step'               => 100,
                'max'                => 5000,
                'default'            => 500,
                'description'        => esc_html__( 'Selection Duration in milliseconds. Min 100 and Max 10000.', 'adv-heading-elementor' ),
                'frontend_available' => true,
                'condition'    => [
                    'animation_type'    => 'letters type',
                ],
            ]
        );

        $this->add_control(
            'type_anim_delay',
            [
                'label'              => esc_html__( 'Type Animation Delay', 'adv-heading-elementor' ),
                'type'               => Controls_Manager::NUMBER,
                'min'                => 100,
                'step'               => 100,
                'max'                => 5000,
                'default'            => 1300,
                'description'        => esc_html__( 'Type Animation Delay in milliseconds. Min 100 and Max 10000.', 'adv-heading-elementor' ),
                'frontend_available' => true,
                'condition'    => [
                    'animation_type'    => 'letters type',
                ],
            ]
        );

        $this->add_control(
            'bar_anim_delay',
            [
                'label'              => esc_html__( 'Bar Animation Delay', 'adv-heading-elementor' ),
                'type'               => Controls_Manager::NUMBER,
                'min'                => 100,
                'step'               => 100,
                'max'                => 10000,
                'default'            => 3800,
                'description'        => esc_html__( 'Bar Animation Delay in milliseconds. Min 100 and Max 10000.', 'adv-heading-elementor' ),
                'frontend_available' => true,
                'condition'    => [
                    'animation_type'    => 'loading-bar',
                ],
            ]
        );

        $this->add_control(
            'bar_waiting',
            [
                'label'              => esc_html__( 'Bar Waiting Time', 'adv-heading-elementor' ),
                'type'               => Controls_Manager::NUMBER,
                'min'                => 100,
                'step'               => 100,
                'max'                => 10000,
                'default'            => 800,
                'description'        => esc_html__( 'Bar Waiting Time in milliseconds. Min 100 and Max 10000.', 'adv-heading-elementor' ),
                'frontend_available' => true,
                'condition'    => [
                    'animation_type'    => 'loading-bar',
                ],
            ]
        );

        $this->end_controls_section();

        // Common Styles
        $this->start_controls_section(
            'section_coming_text_style',
            [
                'label' => esc_html__( 'Common Styles', 'adv-heading-elementor' ),
                'tab'   => Controls_Manager::TAB_STYLE,
            ]
        );

        $this->add_group_control(
            Group_Control_Typography::get_type(),
            [
                'name'     => 'common_typography',
                'label'    => esc_html__( 'Typography', 'adv-heading-elementor' ),
                'default'  => [
                    'font_size' => [''],
                ],
                'selector' => '{{WRAPPER}} .dl-animated-text-wrap, {{WRAPPER}} .dl-animated-text-wrap b, {{WRAPPER}} .dl-animated-text-wrap i, {{WRAPPER}} .dl-animated-text-wrap em',
            ]
        );

        $this->add_control(
            'common_color',
            [
                'label'     => esc_html__( 'Color', 'adv-heading-elementor' ),
                'type'      => Controls_Manager::COLOR,
                'selectors' => [
                    '{{WRAPPER}} .dl-animated-text-wrap' => 'color: {{VALUE}}',
                ],
            ]
        );

        $this->add_group_control(
            Group_Control_Text_Shadow::get_type(),
            [
                'name'     => 'common_shadow',
                'label'    => esc_html__( 'Text Shadow', 'adv-heading-elementor' ),
                'selector' => '{{WRAPPER}} .dl-animated-text-wrap',
            ]
        );

        $this->add_control(
            'loading_bar_color',
            [
                'label'     => esc_html__( 'Loading Bar Color', 'adv-heading-elementor' ),
                'type'      => Controls_Manager::COLOR,
                'selectors' => [
                    '{{WRAPPER}} .dl-animated-text-wrap.loading-bar .dl-animated-text::after' => 'background: {{VALUE}}',
                ],
                'condition' => [
                    'animation_type' => 'loading-bar',
                ],
            ]
        );

        $this->add_control(
            'cursor_color',
            [
                'label'      => esc_html__( 'Cursor Color', 'adv-heading-elementor' ),
                'type'       => Controls_Manager::COLOR,
                'selectors'  => [
                    '{{WRAPPER}} .dl-animated-text-wrap.clip .dl-animated-text::after' => 'background-color: {{VALUE}}',
                    '{{WRAPPER}} .dl-animated-text-wrap.type .dl-animated-text::after' => 'background-color: {{VALUE}}',
                ],
                'conditions' => [
                    'relation' => 'or',
                    'terms'    => [
                        [
                            'name'     => 'animation_type',
                            'operator' => '==',
                            'value'    => [
                                'clip',
                            ],
                        ],
                        [
                            'name'     => 'animation_type',
                            'operator' => '==',
                            'value'    => [
                                'letters type',
                            ],
                        ],
                    ],
                ],
            ]
        );

        $this->add_control(
            'select_text_color',
            [
                'label'     => esc_html__( 'Select Text Color', 'adv-heading-elementor' ),
                'type'      => Controls_Manager::COLOR,
                'selectors' => [
                    '{{WRAPPER}} .dl-animated-text-wrap.type .dl-animated-text.selected' => 'background-color: {{VALUE}}',
                ],
                'condition' => [
                    'animation_type' => 'letters type',
                ],
            ]
        );

        $this->end_controls_section();

        // Before Text
        $this->start_controls_section(
            'section_before_text',
            [
                'label' => esc_html__( 'Before Text', 'adv-heading-elementor' ),
                'tab'   => Controls_Manager::TAB_STYLE,
            ]
        );

        $this->add_group_control(
            Group_Control_Typography::get_type(),
            [
                'name'     => 'before_text_typography',
                'label'    => esc_html__( 'Typography', 'adv-heading-elementor' ),
                'exclude'  => [
                    'font_size',
                    'line_height',
                ],
                'selector' => '{{WRAPPER}} .dl-animated-text-wrap .dl-animated-before-text',
            ]
        );

        $this->add_control(
            'before_text_color',
            [
                'label'     => esc_html__( 'Color', 'adv-heading-elementor' ),
                'type'      => Controls_Manager::COLOR,
                'selectors' => [
                    '{{WRAPPER}} .dl-animated-text-wrap .dl-animated-before-text' => 'color: {{VALUE}}',
                ],
            ]
        );

        $this->add_group_control(
            Group_Control_Text_Shadow::get_type(),
            [
                'name'     => 'before_text_shadow',
                'label'    => esc_html__( 'Text Shadow', 'adv-heading-elementor' ),
                'selector' => '{{WRAPPER}} .dl-animated-text-wrap .dl-animated-before-text',
            ]
        );

        $this->end_controls_section();

        // After Text
        $this->start_controls_section(
            'section_after_text',
            [
                'label' => esc_html__( 'After Text', 'adv-heading-elementor' ),
                'tab'   => Controls_Manager::TAB_STYLE,
            ]
        );

        $this->add_group_control(
            Group_Control_Typography::get_type(),
            [
                'name'     => 'after_text_typography',
                'label'    => esc_html__( 'Typography', 'adv-heading-elementor' ),
                'exclude'  => [
                    'font_size',
                    'line_height',
                ],
                'selector' => '{{WRAPPER}} .dl-animated-text-wrap .dl-animated-after-text',
            ]
        );

        $this->add_control(
            'after_text_color',
            [
                'label'     => esc_html__( 'Color', 'adv-heading-elementor' ),
                'type'      => Controls_Manager::COLOR,
                'selectors' => [
                    '{{WRAPPER}} .dl-animated-text-wrap .dl-animated-after-text' => 'color: {{VALUE}}',
                ],
            ]
        );

        $this->add_group_control(
            Group_Control_Text_Shadow::get_type(),
            [
                'name'     => 'after_text_shadow',
                'label'    => esc_html__( 'Text Shadow', 'adv-heading-elementor' ),
                'selector' => '{{WRAPPER}} .dl-animated-text-wrap .dl-animated-after-text',
            ]
        );

        $this->end_controls_section();

        // Animated Text Style
        $this->start_controls_section(
            'section_custom_border',
            [
                'label' => esc_html__( 'Animated Text', 'adv-heading-elementor' ),
                'tab'   => Controls_Manager::TAB_STYLE,
            ]
        );

        $this->add_group_control(
            Group_Control_Typography::get_type(),
            [
                'name'     => 'animated_text_typography',
                'label'    => esc_html__( 'Typography', 'adv-heading-elementor' ),
                'exclude'  => [
                    'font_size',
                    'line_height',
                ],
                'selector' => '{{WRAPPER}} .dl-animated-text-wrap .dl-animated-text b,{{WRAPPER}} .dl-animated-text-wrap .dl-animated-text i,{{WRAPPER}} .dl-animated-text-wrap .dl-animated-text em',
            ]
        );

        $this->add_control(
            'animated_text_color',
            [
                'label'     => esc_html__( 'Color', 'adv-heading-elementor' ),
                'type'      => Controls_Manager::COLOR,
                'selectors' => [
                    '{{WRAPPER}} .dl-animated-text-wrap .dl-animated-text b,{{WRAPPER}} .dl-animated-text-wrap .dl-animated-text i,{{WRAPPER}} .dl-animated-text-wrap .dl-animated-text em' => 'color: {{VALUE}}',
                ],
            ]
        );

        $this->add_group_control(
            Group_Control_Text_Shadow::get_type(),
            [
                'name'     => 'animated_text_shadow',
                'label'    => esc_html__( 'Text Shadow', 'adv-heading-elementor' ),
                'selector' => '{{WRAPPER}} .dl-animated-text-wrap b',
            ]
        );

        $this->end_controls_section();
    }

    protected function render() {
        $settings       = $this->get_settings_for_display();
        $animation_type = $settings['animation_type'];
        $data = [
			'anim_type' => $settings['animation_type'],
			'delay' => ! empty( $settings['animation_delay'] ) ? $settings['animation_delay'] : 2500,
			'clip_duration' => ! empty( $settings['clip_anim_duration'] ) ? $settings['clip_anim_duration'] : 600,
			'clip_delay' => ! empty( $settings['clip_anim_delay'] ) ? $settings['clip_anim_delay'] : 1500,
			'type_letter_delay' => ! empty( $settings['type_letter_delay'] ) ? $settings['type_letter_delay'] : 150,
			'selection_duration' => ! empty( $settings['selection_duration'] ) ? $settings['selection_duration'] : 500,
			'type_anim_delay' => ! empty( $settings['type_anim_delay'] ) ? $settings['type_anim_delay'] : 1300,
			'bar_anim_delay' => ! empty( $settings['bar_anim_delay'] ) ? $settings['bar_anim_delay'] : 3800,
			'bar_waiting' => ! empty( $settings['bar_waiting'] ) ? $settings['bar_waiting'] : 800,
		];
        $this->add_render_attribute(
			'dl-animated',
			[
				'class' => [ 'dl-animated-text-wrap cd-headline', $animation_type ],
				'data-settings' => wp_json_encode( $data )
			]
		);

        $animated_text = '';

        if ( $settings['before_text'] ) {
            $this->add_render_attribute( 'dl-animated', 'class', 'dl-animated-has-before-text' );
            $animated_text .= sprintf( '<span class="dl-animated-before-text">%s</span>', esc_html( $settings['before_text'] ) );
        }

        if ( $settings['animated_text'] && is_array( $settings['animated_text'] ) ) {
            $animated_animation_text = '';

            foreach ( $settings['animated_text'] as $key => $item ) {
                $animated_animation_text .= sprintf(
                    '<b class="elementor-repeater-item-%s">%s</b>',
                    esc_attr( $item['_id'] . ( $key === 0 ? ' is-visible' : '' ) ),
                    esc_html( $item['text'] )
                );
            }

            $animated_text .= sprintf(
                ' <span class="dl-animated-text cd-words-wrapper">%s</span>',
                $animated_animation_text
            );
        }

        if ( $settings['after_text'] ) {
            $this->add_render_attribute( 'dl-animated', 'class', 'dl-animated-has-after-text' );
            $animated_text .= sprintf( ' <span class="dl-animated-after-text">%s</span>', esc_html( $settings['after_text'] ) );
        }

        printf(
            '<%1$s %2$s>%3$s</%1$s>',
            tag_escape( $settings['html_tag'] ),
            $this->get_render_attribute_string( 'dl-animated' ),
            $animated_text
        );
    }
}